// ***************************************************************************
// Implementation System Includes
// ***************************************************************************

#include <Box.h>
#include <Button.h>
#include <Directory.h>
#include <Entry.h>
#include <Message.h>
#include <Messenger.h>
#include <Screen.h>
#include <TextControl.h>
#include <Window.h>

#include <fnmatch.h>

// ***************************************************************************
// Implementation Project Includes
// ***************************************************************************

#include "Main.h"
#include "SelectiveWindow.h"

// ***************************************************************************
// Implementation Constant Definitions
// ***************************************************************************

/** The messages that the views send to the window. */
enum {
  MSG_SELECT = 1000,
};

// ***************************************************************************
// Implementation Type Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Variable Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Declarations
// ***************************************************************************

// ***************************************************************************
// Global Variable Definitions
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions: Lifecycle
// ***************************************************************************

// ===========================================================================
// Constructor
// ===========================================================================

tSelectiveWindow::tSelectiveWindow(entry_ref aDirRef, BMessage* apMessage) 
  : BWindow(BRect(0, 0, 1, 1),
            "Selective",
            B_TITLED_WINDOW,
            B_NOT_RESIZABLE)
{
  // Initialize attributes.
  mTrackerDirRef = aDirRef;
  mpTrackerMessage = apMessage;

  // Create the selection pattern entry field.
  mpPatternText = new BTextControl(BRect(0, 0, 0, 1),
          "Pattern",
          0,                           // No label.
          0,                           // No initial text.
          new BMessage(MSG_SELECT),
          B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP,
          B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE | B_NAVIGABLE); 

  mpPatternText->TextView()->SetMaxBytes(1000); 
  mpPatternText->ResizeToPreferred();

  // Create the "select" button.
  BButton* lpSelectButton = new BButton(BRect(0, 1, 80, 1),
          "SelectButton",
          "Select",
          new BMessage(MSG_SELECT),
          B_FOLLOW_RIGHT);
          
  lpSelectButton->ResizeToPreferred();  

  // Create the "cancel" button.
  BButton* lpCancelButton = new BButton(BRect(0, 1, 80, 1), 
          "CancelButton",
          "Cancel",
          new BMessage(B_QUIT_REQUESTED),
          B_FOLLOW_RIGHT);

  lpCancelButton->ResizeToPreferred();  

  // Now all views have been created, we can calculate how large the window
  // will be.
  tFloat lWidth  = 8
                 + lpCancelButton->Frame().Width()
                 + 8
                 + lpSelectButton->Frame().Width()
                 + 8;
  
  tFloat lHeight = 8
                 + mpPatternText->Frame().Height()
                 + 8 
                 + lpCancelButton->Frame().Height()
                 + 8;

  // Resize the window to its final size.
  ResizeTo(lWidth, lHeight);  
  
  // Center the window on the screen.
  BRect lScreenRect = BScreen(this).Frame();
  MoveTo((lScreenRect.Width()-lWidth)/2, (lScreenRect.Height()-lHeight)/2);
  
  // Create the background box.
  BBox* lpBox  = new BBox(BRect(0, 0, lWidth, lHeight),
          B_EMPTY_STRING, 
          B_FOLLOW_ALL, 
          B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE_JUMP, 
          B_PLAIN_BORDER); 

  AddChild(lpBox);

  // Add the selection pattern entry field to the background box.
  mpPatternText->MoveTo(8, 8);
  lpBox->AddChild(mpPatternText);
  mpPatternText->ResizeBy(lWidth - 16, 0);
  mpPatternText->MakeFocus(TRUE);

  // Add the "select" button to the background box.
  lpSelectButton->MoveTo(
          lWidth  - lpSelectButton->Frame().Width()  - 8, 
          lHeight - lpSelectButton->Frame().Height() - 8);
  lpBox->AddChild(lpSelectButton); 

  // Add the "cancel" button to the background box.
  lpCancelButton->MoveTo(
          8, 
          lHeight - lpCancelButton->Frame().Height() - 8);
  lpBox->AddChild(lpCancelButton); 
  
  // Finally show the window.
  Show(); 
}

// ***************************************************************************
// PUBLIC Member Function Definitions: Operators
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions: Operations
// ***************************************************************************

// ===========================================================================
// MessageReceived
// ===========================================================================

void
tSelectiveWindow::MessageReceived(BMessage* apMessage)
{
  // Figure out what kind of message was received.
  switch(apMessage->what) {
  
    // Make the selection. ---------------------------------------------------
    case MSG_SELECT: {

      // Hide our window.
      Hide();
      
      // Get a messenger to the Poses property of the Tracker window that the 
      // user called us from.
      BMessenger lPoses;
      if (mpTrackerMessage->FindMessenger("TrackerViewToken", 
                                          &lPoses) == B_OK) {
        if (lPoses.IsValid()) {

          /*!
           * [BUG] When no files match, the old selection isn't deleted.
           *
           * We need to tell the Poses property to clear its current
           * selection before making a new selection. This is necessary 
           * because the Tracker will not automatically clear the old 
           * selection if the new selection message doesn't contain any
           * entry_refs. Unfortunately, I don't seem to be able to delete 
           * the selection, no matter what I try.
           *
           * @date Saturday, 3 April 1999
           */
          
          // Create a message that tells the Poses to make a new selection.
          BMessage lSelectionMsg(B_SET_PROPERTY);
          lSelectionMsg.AddSpecifier("Selection");

          // Loop through the entries inside the directory.
          BDirectory lDir(&mTrackerDirRef);
          entry_ref lCurrentRef;
          while (lDir.GetNextRef(&lCurrentRef) == B_OK) {

            // If the current entry matches the wildcard pattern, then we 
            // will add its entry_ref to the selection message.
            if (fnmatch(mpPatternText->Text(), 
                        lCurrentRef.name, 
                        FNM_NOESCAPE|FNM_CASEFOLD) != FNM_NOMATCH) {
        
              lSelectionMsg.AddRef("data", &lCurrentRef);
            }
          }

          // Send the selection message to the Poses property.
          lPoses.SendMessage(&lSelectionMsg);
        }
      }
      
      // Commit suicide.
      PostMessage(B_QUIT_REQUESTED);
      break;
    }

    // Let the original BWindow handle the message. --------------------------
    default: {
      BWindow::MessageReceived(apMessage);
      break;
    }
  }
}

// ***************************************************************************
// PUBLIC Member Function Definitions: Access
// ***************************************************************************

// ***************************************************************************
// PUBLIC Member Function Definitions: Inquiry
// ***************************************************************************

// ***************************************************************************
// PROTECTED Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// PRIVATE Member Function Definitions
// ***************************************************************************

// ***************************************************************************
// Helper Function Definitions
// ***************************************************************************

// ***************************************************************************
// Global Function Definitions
// ***************************************************************************

// ***************************************************************************
// Implementation Function Definitions
// ***************************************************************************
